/* Low level initialization for the STR966e in the comstrick  						
/* What needs to be done:
	Set the interrupt vectors
	Set up the stacks for the various modes	
	call main

Expected processor modes are IRQ mode and System mode.
*/
.set  	SCU_BASE,		0x5C002000
.set	SCU_SCR0,		0x5C002034
.set 	RAM_END, 		0x40018000		/* str9 top of RAM							*/

.set  	UNDEF_STACK_SIZE, 	0x00000010		/* stack for "undefined instruction" interrupts is a minimal 4 bytes   	*/
.set  	ABT_STACK_SIZE, 	0x00000010		/* stack for "abort" interrupts is also a minimal 4 bytes               */
.set  	FIQ_STACK_SIZE,		0x00000010		/* stack for "FIQ" interrupts  is a minimal 4 bytes         		*/
.set  	IRQ_STACK_SIZE,		0X00000200		/* stack for "IRQ" normal interrupts is 512 bytes    			*/
.set  	SVC_STACK_SIZE,		0x00000084		/* stack for "SVC" supervisor mode is 128 bytes  			*/
.set  	SYS_STACK_SIZE,		0x00000100		/* stack size for system mode is 256 bytes - perhaps exessive		*/


.set USR_MODE,			0x10
.set FIQ_MODE,			0x11
.set IRQ_MODE, 			0x12
.set SVC_MODE, 			0x13
.set ABT_MODE, 			0x17
.set UND_MODE, 			0x1B
.set SYS_MODE, 			0x1F
.set I_BIT, 			0x80    		/* IRQ disable bit */
.set F_BIT, 			0x40    		/* FIQ disable bit */

/* setup the ISR's first - including reset address	*/
INTERRUPT_VECTOR_JUMP_TABLE:
		ldr     PC, =RESET_ISR        
                ldr     PC, =UNDEF_ISR
                ldr     PC, =SWI_ISR
                ldr     PC, =PABT_ISR
                ldr     PC, =DABT_ISR
                nop							
                ldr     PC, [PC,#-0xFF0]	
                ldr     PC, =FIQ_ISR
		nop
		nop
		nop
		nop
		nop
		nop
		nop		
		nop
		nop
		nop
		nop
		nop
		nop
		nop

RESET_ISR:
/* set the RAM size to 96k */
	ldr	r1,	=SCU_SCR0
	mov	r0,	#0x18
	str	r0,	[r1]
/* need to set up the FMI registers to set ROM size to 512k */
	mov   r0,	#0xf
	mov   r1,	#0x54000000
	nop
	nop
	str  r0,[ r1 ]
/* end FMI setup */
/* set up the stacks for the vaious modes.  Will not switch to user mode as foreground processing will take place in system mode */

	ldr   r0,	=RAM_END
	msr   CPSR_c, 	#UND_MODE | I_BIT | F_BIT 	/* Undefined Instruction Mode  */
	mov   sp, r0	

	sub   r0, r0, 	#UNDEF_STACK_SIZE
    	msr   CPSR_c, 	#ABT_MODE | I_BIT | F_BIT 	/* Abort Mode */
    	mov   sp, r0

    	sub   r0, r0, 	#ABT_STACK_SIZE
    	msr   CPSR_c, 	#FIQ_MODE | I_BIT | F_BIT 	/* FIQ Mode */
    	mov   sp, r0	

   	sub   r0, r0, 	#FIQ_STACK_SIZE
    	msr   CPSR_c, 	#IRQ_MODE | I_BIT | F_BIT 	/* IRQ Mode */
    	mov   sp, r0

    	sub   r0, r0, 	#IRQ_STACK_SIZE
    	msr   CPSR_c, 	#SVC_MODE | I_BIT | F_BIT	/* Supervisor Mode */
    	mov   sp, r0

    	sub   r0, r0, 	#SVC_STACK_SIZE
    	msr   CPSR_c, 	#SYS_MODE | I_BIT | F_BIT 	/* System Mode */
    	mov   sp, r0

/* Some global and static variables may need initialization so copy the relevant data from
   text section to the data section (in ram) 						*/	
	/* copy .data section (Copy from ROM to RAM) */
        ldr     R1, =_etext
        ldr     R2, =_data
	ldr     R3, =_edata
dinit:  cmp     R2, R3
        ldrlo   R0, [R1], #4
        strlo   R0, [R2], #4
        blo     dinit

	/* Clear .bss section  */
        mov     R0, #0
        ldr     R1, =__bss_start__
        ldr     R2, =__bss_end__
zinit:	cmp     R1, R2
        strlo   R0, [R1], #4
        blo     zinit
/******************************************************************************************
***	LOOK HERE!!!									***
***	This is where the jump to main happens!						***
******************************************************************************************/ 
	/* Call main */
        b       main	

/* Interrupt service routines defined below.  These are simply placeholders.  If one of these 
   interrupts occurs then the processor will enter an endless loop - may help debugging 	*/
UNDEF_ISR:
	b	UNDEF_ISR
	nop
	nop
	nop
	nop

SWI_ISR:
	b	SWI_ISR
	nop
	nop
	nop
	nop

PABT_ISR:
	b	PABT_ISR
	nop
	nop
	nop
	nop

DABT_ISR:	
	b	DABT_ISR
	nop
	nop
	nop
	nop

FIQ_ISR:	
	b	FIQ_ISR
	nop
	nop
	nop
	nop

.end	

